#pragma once

#define PEERID_PREFIX             "-TR3000-"
#define USERAGENT_PREFIX          "3.00"
#define VCS_REVISION              "bb6b5a062e"
#define VCS_REVISION_NUM          bb6b5a062e
#define SHORT_VERSION_STRING      "3.00"
#define LONG_VERSION_STRING       "3.00 (bb6b5a062e)"
#define VERSION_STRING_INFOPLIST  3.00
#define BUILD_STRING_INFOPLIST    14714.3.0
#define MAJOR_VERSION             3
#define MINOR_VERSION             0
#define TR_STABLE_RELEASE         1
